from .func import str2date, date2str, mkdir, dateminus, dateplus, showeveryday, splitlist, cartesian, \
    find_pd, chineseloto, list_inter, list_union, list_dif
from .nlp import getsim
from .plt import Stackedbar, multiplebar, pieplt
from .ml import  MachineLearningClassify


from .autoinstall import install_package



